package mtv.core;

import java.util.ArrayList;
import java.util.Iterator;


/**
 * Classe reprsentant une leon. Une leon est compose d'un certains nombres de mots de
 * vocabulaire anglais <=> franais.
 * Par souci de simplicit, j'ai choisi de reprsenter une leon par un ArrayList<ArrayList<String>> 
 * (deux dimensions) contenant les ArrayList<String> respectifs des mots anglais et francais.
 * Un choix de structure de donne de type HashMap ou Hashtable aurait galement pu convenir, ou
 * aurait mme t plus adapte  la situation, mais vu qu'il est possible d'tre interrog dans les 
 * sens anglais => franais et franais=>anglais, le sytme associatif cl => valeur des HashMap (ou Hashtable) 
 * aurait complexifi lgrement les choses.
 *
 * @author Christopher Henard
 * @version 26/10/2008
 */

public class Lesson{

    //CONSTANTES--------------------------------------------------
    public static final int LANG_COUNT = 2; //Anglais et Franais
    public static final int ENG = 0;
    public static final int FR = 1;



    //CHAMPS------------------------------------------------------
    private String name;
    private ArrayList<ArrayList<String>> words;
    private ArrayList<String> engWords, frWords;
    private boolean checked; //Une leon sera dans le test ou non.



    //CONSTRUCTEUR-------------------------------------------------
    /**
     * Construit une leon  partir de son nom.
     * @param name le nom de la leon.
     */
    public Lesson(String name){
	this.name = name;
	words = new ArrayList<ArrayList<String>>(LANG_COUNT);
	engWords = new ArrayList<String>();
	frWords = new ArrayList<String>();
	words.add(engWords);
	words.add(frWords);
	checked = true; //par dfaut, une leon sera teste.
    }

    /**
     * Construit une leon  partir d'une autre (copie).
     * @param l la leon  copier.
     */
    public Lesson(Lesson l){
	name = l.name;
	words = l.words;
	engWords = l.engWords;
	frWords = l.frWords;
	checked = l.checked; 
    }



    //METHODES D'OBSERVATION ET DE TRANSFORMATION------------------
    /**
     * Retourne le nom de la leon.
     * @return le nom de la leon.
     */
    public String getName(){
	return name;
    }

    /**
     * Retourne les mots de la leon.
     * @return le tableau des mots de la leon.
     */
    public ArrayList<ArrayList<String>> getWords(){
	return words;
    }

    /**
     * Retourne les mots anglais.
     * @return le tableau contenant les mots anglais de la leon.
     */
    public ArrayList<String> getEngWords(){
	return getWords().get(ENG);
    }
    
    /**
     * Retourne les mots franais.
     * @return le tableau contenant les mots franais de la leon.
     */
    public ArrayList<String> getFrWords(){
	return getWords().get(FR);
    }

    /**
     * Retourne le nombre de mots de vocabulaires de la leon.
     * @return le nombre de mots de vocabulaires de la leon.
     */
    public int getLength() {	
	return getEngWords().size();
    }

    /**
     * Retourne le mot de vocabulaire de langue lang et d'indice index.
     * @param lang la langue (ENG ou FR).
     * @param index l'indice du mot dans la leon.
     * @return le mot de vocabulaire de langue lang et d'indice index.
     */
    public String getWord(int lang, int index){
	    return (lang == ENG)?getEngWords().get(index):getFrWords().get(index);
    }

    /**
     * Teste si la leon sera dans le test ou non.
     * @return true si la leon sera dans le test.
     */
    public boolean isChecked(){
	return checked;
    }

    /**
     * Teste si le mot saisi (answer) correspond au mot de langue oppose d'indice index.
     * @param lang la langue du mot oppos  answer.
     * @param index l'indice du mot oppos  answer.
     * @param answer le mot saisi  comparer.
     * @return true si answer correspond au mot de langue oppose d'indice index.
     */
    public boolean compare(int lang, int index, String answer){
	if (! answer.equals("\n") && ! answer.equals("") && answer != null){
	    String word = getWord(lang,index).toLowerCase();
	    answer = answer.toLowerCase();
	    boolean b1, b2, b3;
	    b1 = b2 = b3 = false;
	    // 	word.replace('','e'); word.replace('','e'); word.replace('','e'); word.replace('','a');
	    // 	word.replace('','a'); word.replace('','i'); word.replace('','i'); word.replace('','o');
	    // 	word.replace('','o'); word.replace('','u'); word.replace('','u'); 
	    // 	answer.replace('','e'); answer.replace('','e'); answer.replace('','e'); answer.replace('','a');
	    // 	answer.replace('','a'); answer.replace('','i'); answer.replace('','i'); answer.replace('','o');
	    // 	answer.replace('','o'); answer.replace('','u'); answer.replace('','u'); 
	    if (word.contains(",")){
		b1 = word.substring(0,word.indexOf(',')).equals(answer);
		b2 = word.substring(word.lastIndexOf(',')+2, word.length()).equals(answer);
	    }
	    if (word.contains("(") && word.contains(")")){
		b3 = word.substring(0,word.indexOf('(')).trim().equals(answer);		
	    }	    
	    return word.equals(answer) || word.trim().equals(answer.trim()) || word.replace('-',' ').equals(answer.replace('-',' ')) ||
		b1 || b2 || b3;
	}
	return false;
    }

    /**
     * Teste si la leon ne contient plus de mots.
     * @return true si la leon est vide.
     */
    public boolean isEmpty(){
	return getEngWords().isEmpty();
    }

    /**
     * Retourne un indice alatoire d'un mot de la leon.
     * @return un entier correspondant  l'indice d'un mot dans la leon.
     */
    public int getRandomIndex(){
	return (int)(Math.random() * getLength());
    }

    /**
     * Retourne une reprsentation de la leon.
     * @return une chaine de caractres reprsentant la leon.
     */
    public String toString(){
	String s = "";
	Iterator eng = getEngWords().iterator();
	Iterator fr = getFrWords().iterator();
	while (eng.hasNext() && fr.hasNext()){
	    s += eng.next()+ "=" + fr.next()+"\n";
	}
	return s;
    }

    /**
     * Renomme la leon.
     * @param name le nom de la leon.
     */
    public void setName(String name){
	this.name = name;
    }

    /**
     * Spcifie les mots de la leon.
     * @param words les mots de la leon.
     */
    public void setWords(ArrayList<ArrayList<String>> words){
	this.words = words;
    }

    /**
     * Spcifie les mots anglais de la leon.
     * @param engWords les mots anglais de la leon.
     */
    public void setEngWords(ArrayList<String> engWords){
	this.engWords = engWords;
	getWords().set(ENG, getEngWords());	
    }

    /**
     * Spcifie les mots francais de la leon.
     * @param frWords les mots francais de la leon.
     */
    public void setFrWords(ArrayList<String> frWords){
	this.frWords = frWords;
	getWords().set(FR, getFrWords());	
    }

    /**
     * Spcifie le mot de langue lang et d'indice index dans la leon.
     * @param lang la langue (ENG ou FR).
     * @param index l'indice du mot dans la leon.
     * @param word le mot  spcifier.
     */
    public void setWord(int lang, int index, String word){
	if (lang == ENG)
	    getEngWords().set(index,word);
	else
	     getFrWords().set(index,word);	    
    }

    /**
     * Ajoute un mot  la leon.
     * @param engWord le mot en anglais.
     * @param frWord le mot en franais.
     */
    public void addWord(String engWord, String frWord){
	getEngWords().add(engWord);
	getFrWords().add(frWord);
    }

    /**
     * Retire le mot d'indice index de la leon.
     * @param index l'indice du mot  retirer.
     */
    public void removeWord(int index){
	getEngWords().remove(index);
	getFrWords().remove(index);
    }

    /**
     * Inclu ou retire la leon d'un test.
     * @param b spcifie si la leon sera incluse dans le test.
     */
    public void check(boolean b){
	checked = b;
    }	

}//class Lesson
    
